/***
 * uGASP: a gaming services platform for mobile multiplayer games.
 * GASP Demonstrator: a gasp demonstration game.
 *
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Gabriel Adgeg
 */
package org.ugasp.impl.games.demonstrator.test;

import java.io.File;
import java.io.IOException;

import org.ugasp.eventmodel.listener.OnEvent;
import org.ugasp.impl.gamelogic.GameLogicImpl;
import org.ugasp.impl.games.demonstrator.protocol.moods.CustomTypesImpl;
import org.ugasp.impl.games.demonstrator.protocol.moods.FirstMooDSObject;
import org.ugasp.logging.Logger;

import org.ugasp.protocol.rmai.RMAIUtils;


/**
 * This class represents the game logic on the server side.
 * Each instance of your application is represented by an instance of this class, e.g.
 * if application model is set to "manual" and N GASPClients call the createApplicationInstance method,
 * uGASP platform will create N instances of this class.
 * 
 * Behavior of game logic on the reception of in game events have to be put in several onEvent methods.
 * 
 * Properties are linked with this game logic, and can be set in metadata.xml 
 * (located in test-demonstrator-gamelogic/src/resources):
 * <ul>
 * <li>appID is the identifier of your application, which is needed when clients want to connect it</li>
 * <li>model is the model of instance for your application, which can be chosen between "manual", "auto" and "permanent"</li>
 * <li>timeout is the time from which a client which stopped to communicate with the server is considered as disconnected, 
 * and thus raise a QuitEvent</li>
 * </ul>
 * 
 * @see GASPClient.createApplicationInstance
 * @author PELLERIN Romain (Ubidreams, romain.pellerin@ubidreams.com )
 * @author ADGEG Gabriel (Telecom Sud Paris, gabriel.adgeg@telecom-sudparis.eu)
 */
public class GameLogicDemonstratorForTests extends GameLogicImpl implements OnEvent{
	
	protected static final String RMAI_SERIALIZABLE_OBJECTS_FOLDER_NAME = "org.ugasp.impl.games.demonstrator.protocol.rmai.objects";
	protected static final String STRING_TO_TEST_MooDS = "MooDS works!";
	
	private 	RMAIUtils 	rmaiUtils;// iPojo injected field
	private 	Logger 		logger;
	
    protected 	File 		gameLogicTestFile;
    
	public GameLogicDemonstratorForTests(){
		// set the name of CustomType implementation
		setCustomTypes(new CustomTypesImpl());
		// set the folder where RMAISerializable object are located
		rmaiUtils.setRMAISerializableObjectsFolder(RMAI_SERIALIZABLE_OBJECTS_FOLDER_NAME);
		if(logger != null){
			logger.log(Logger.INFO,this.getClass(),"uGASP Test Game Logic Demonstrator instantiated");
		}else{
			System.out.println("Error occured on the retrieval of logging service. Make sure it is properly started.");
		}	
	}
	
	@Override
	public Logger getLogger() {
		return logger;
	}
	
	@Override
	protected RMAIUtils getRMAIUtils() {
		return rmaiUtils;
	}

	@Override
	protected Object newRMAIGameLogicManager(short senderASID, short recipientASID) {
		return new RMAIGameLogicManagerImpl(this,senderASID,recipientASID);
	}
	
	
	/**
	 * Treatment of MooDS object received.
	 * You have to insert you treatment for each object after checking
	 * from which class Object o is an instance.
	 *
 	 *@param senderActorSessionID the actorSessionID of the sender of datas
	 *@param Object	the object to treat. 
	 */
	protected void treatMooDSMessage(Object moodsObject, short senderActorSessionID){
		if(moodsObject instanceof FirstMooDSObject){
			FirstMooDSObject fmo = (FirstMooDSObject)moodsObject;
			// store message on the DataOutputStream used to store RMAI
			try{
				RMAIGameLogicManagerImpl.storeMooDSMessage(fmo.getText());
			}catch(IOException e){
				e.printStackTrace();
			}
		}
	}
	
}
